/* $Id: uves_cal_ronbias.c,v 1.6 2008/08/14 13:51:08 amodigli Exp $
 *
 * This file is part of the DETMON Pipeline
 * Copyright (C) 2002,2003 European Southern Observatory
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */

/*
 * $Author: amodigli $
 * $Date: 2008/08/14 13:51:08 $
 * $Revision: 1.6 $
 * $Name: uves-4_9_1 $
 */
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

/*----------------------------------------------------------------------------
                             Includes
 ----------------------------------------------------------------------------*/
#include <stdbool.h>

#include <cpl.h>

#include "irplib_detmon.h"
#include "uves_plugin.h"


#include "uves_msg.h"
#include "uves_error.h"
#include "uves_utils.h"
#include "uves_utils_wrappers.h"
/* Define here the DO.CATG keywords */
#define REGEXP "ARCFILE|MJD-OBS|ESO TPL ID|DATE-OBS|ESO DET DIT|ESO DET NDIT"
#define RECIPE_NAME "uves_cal_ronbias"
#define UVES_RONBIAS_RAW(it) ((it) == 0 ? "BIAS_RED" : \
                              (it) == 1 ? "BIAS_BLUE" : \
                                 "???")

#define UVES_RONBIAS_MASTER_BIAS(it) ((it) == 0 ? "MASTER_BIAS_RED" : \
                                       (it) == 1 ? "MASTER_BIAS_BLUE" : "???")

#define UVES_RONBIAS_INT_XSTRUCT(it) ((it) == 0 ? "INT_XSTRUCT_RED" : \
                                      (it) == 1 ? "INT_XSTRUCT_BLUE" : "???")


#define UVES_RONBIAS_INT_YSTRUCT(it) ((it) == 0 ? "INT_YSTRUCT_RED" : \
                                      (it) == 1 ? "INT_YSTRUCT_BLUE" : "???")


#define UVES_RONBIAS_SYNTH(it) ((it) == 0 ? "SYNTH_RED" : \
                                (it) == 1 ? "SYNTH_BLUE" : "???")


#define UVES_RONBIAS_BP_MAP_HP(it) ((it) == 0 ? "BP_MAP_HP_RED" : \
                                    (it) == 1 ? "BP_MAP_HP_BLUE" : "???")


#define UVES_RONBIAS_BP_MAP_CP(it) ((it) == 0 ? "BP_MAP_CP_RED" : \
                                    (it) == 1 ? "BP_MAP_CP_BLUE" : "???")


#define UVES_RONBIAS_BP_MAP_DP(it) ((it) == 0 ? "BP_MAP_DP_RED" : \
                                    (it) == 1 ? "BP_MAP_DP_BLUE" : "???")



#define UVES_PIX2PIX CPL_TRUE
#define UVES_EXTS_RED -1
/*----------------------------------------------------------------------------
				Functions prototypes
 ----------------------------------------------------------------------------*/

UVES_IRPLIB_RECIPE_DEFINE(uves_cal_ronbias, 
                     UVES_BINARY_VERSION,
                     irplib_detmon_ronbias_fill_parlist_default(recipe->parameters,
                                                                RECIPE_NAME,
                                                                PACKAGE_TARNAME),
                     "Lander de Bilbao", PACKAGE_BUGREPORT, "2008",
                     "RON/Bias recipe",
                     irplib_detmon_ronbias_get_description(RECIPE_NAME,"UVES",
                                                           "BIAS_BLUE or BIAS_RED"));

/*---------------------------------------------------------------------------*/
/*
  @brief    Interpret the command line options and execute the data processing
  @param    frameset    the frames list
  @param    parlist     the parameters list
  @return   0 if everything is ok
 */
/*---------------------------------------------------------------------------*/

static int uves_cal_ronbias(cpl_frameset            * frameset,
                          const cpl_parameterlist * parlist)
{


   int red;
   bool found_bias = false;
   int nbias = 0;

   cpl_error_code error=0;
   int processed=0;
   cpl_parameter* p=NULL;
   cpl_parameterlist* plist=NULL;
   cpl_frameset* bset=NULL;

   check_nomsg(plist=uves_parameterlist_duplicate(parlist));

   for (red = 0; red <= 1; red++){

      found_bias = cpl_frameset_find(frameset, UVES_RONBIAS_RAW(red));
      if(found_bias) {
         check_nomsg(bset=uves_frameset_extract(frameset,UVES_RONBIAS_RAW(red)));
         check_nomsg(nbias=cpl_frameset_get_size(bset));
         uves_msg("found %d biases",nbias);
         if(nbias>1) {
            if(red == 0) {
               //uves_msg_warning("%s data reduction not supported", 
               //                 UVES_RONBIAS_RAW(red));
                
               uves_msg("Update exts param for RED ARM data");
               check_nomsg(p=cpl_parameterlist_find(plist,PACKAGE "." RECIPE_NAME ".exts"));

               cpl_ensure_code(p != NULL, CPL_ERROR_DATA_NOT_FOUND);
               check_nomsg(error = cpl_parameter_set_int(p, UVES_EXTS_RED));
               cpl_ensure_code(!error, error);
               


            }
            processed++;

            cpl_propertylist * master =
               irplib_detmon_fill_prolist(UVES_RONBIAS_MASTER_BIAS(red),
                                          "TYPE", "TECH", CPL_TRUE);

            cpl_propertylist * synth = 
               irplib_detmon_fill_prolist(UVES_RONBIAS_SYNTH(red),   
                                          "TYPE", "TECH", CPL_TRUE);

            cpl_propertylist * bpmhot = 
               irplib_detmon_fill_prolist(UVES_RONBIAS_BP_MAP_HP(red),
                                          "TYPE", "TECH", CPL_TRUE);

            cpl_propertylist * bpmcold = 
               irplib_detmon_fill_prolist(UVES_RONBIAS_BP_MAP_CP(red),
                                          "TYPE", "TECH", CPL_TRUE);

            cpl_propertylist * bpmdev = 
               irplib_detmon_fill_prolist(UVES_RONBIAS_BP_MAP_DP(red),
                                          "TYPE", "TECH", CPL_TRUE);

            check_nomsg(error=
                        irplib_detmon_ronbias(frameset,
                                              parlist,
                                              UVES_RONBIAS_RAW(red),
                                              RECIPE_NAME,
                                              PACKAGE_TARNAME,
                                              REGEXP,
                                              master,
                                              NULL,
                                              NULL,
                                              synth,
                                              bpmhot,
                                              bpmcold,
                                              bpmdev,
                                              PACKAGE "/" PACKAGE_VERSION,
                                              NULL,OPT));


            cpl_propertylist_delete(master);
            cpl_propertylist_delete(synth);
            cpl_propertylist_delete(bpmhot);
            cpl_propertylist_delete(bpmcold);
            cpl_propertylist_delete(bpmdev);

            
         } else {

            uves_msg("You must provide at least 2 frames tagged as %s",
                     UVES_RONBIAS_RAW(red));

         }
         uves_free_frameset(&bset);

      } else {
         uves_msg("You provided no frame tagged as %s",UVES_RONBIAS_RAW(red));
      }
   }


  cleanup:
   uves_free_frameset(&bset);

   if(processed>0) {
      /* Propagate the error, if any */
      cpl_ensure_code(!error, error);
   } else {
      uves_msg("No data has been processed.");
      uves_msg("Pls check if input frame set contains valid tags");
   }



   return CPL_ERROR_NONE;
}


